﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Swashbuckle.AspNetCore.Swagger;
using Swashbuckle.AspNetCore.SwaggerGen;

namespace Agile.Demo1.API
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddMvc().SetCompatibilityVersion(CompatibilityVersion.Version_2_1);
            services.AddMvc().AddJsonOptions(option =>
            {
                //配置大小写问题，默认是首字母小写
                option.SerializerSettings.ContractResolver = new Newtonsoft.Json.Serialization.DefaultContractResolver();
                //配置序列化时时间格式为yyyy-MM-dd HH:mm:ss
                option.SerializerSettings.DateFormatString = "yyyy-MM-dd HH:mm:ss";
            });

            // 配置版本信息
            services.AddApiVersioning(o =>
            {
                o.ReportApiVersions = true;
                o.AssumeDefaultVersionWhenUnspecified = true;
                o.DefaultApiVersion = new ApiVersion(1, 0);
            });

            // 配置Swagger信息
            services.AddSwaggerGen(options =>
            {
                // Specify two versions 
                options.SwaggerDoc("Demo1APIv1",
                    new Info()
                    {
                        Version = "v1",
                        Title = "Demo1 API",
                        Description = "测试接口1",
                        TermsOfService = "测试接口1"
                    });

                options.SwaggerDoc("Demo1APIv2",
                    new Info()
                    {
                        Version = "v2",
                        Title = "Demo1 API",
                        Description = "测试接口1",
                        TermsOfService = "测试接口1"
                    });


                // This call remove version from parameter, without it we will have version as parameter 
                // for all endpoints in swagger UI
                options.OperationFilter<RemoveVersionFromParameter>();

                // This make replacement of v{version:apiVersion} to real version of corresponding swagger doc.
                options.DocumentFilter<ReplaceVersionWithExactValueInPath>();

                // This on used to exclude endpoint mapped to not specified in swagger version.
                // In this particular example we exclude 'GET /api/v2/Values/otherget/three' endpoint,
                // because it was mapped to v3 with attribute: MapToApiVersion("3")
                options.DocInclusionPredicate((docName, apiDesc) =>
                {
                    if (!apiDesc.TryGetMethodInfo(out MethodInfo methodInfo))
                        return false;

                    //不做版本控制的直接返回
                    if (methodInfo.DeclaringType.GetCustomAttributes(true).OfType<ApiVersionAttribute>().Count() == 0)
                    {
                        return true;
                    }

                    //做版本区分的处理下面逻辑
                    var versions = methodInfo.DeclaringType
                        .GetCustomAttributes(true)
                        .OfType<ApiVersionAttribute>()
                        .SelectMany(attr => attr.Versions);

                    var maps = methodInfo.DeclaringType
                        .GetCustomAttributes(true)
                        .OfType<MapToApiVersionAttribute>()
                        .SelectMany(attr => attr.Versions)
                        .ToArray();

                    return versions.Any(v => $"Demo1APIv{v.ToString()}" == docName) && (maps.Length == 0 || maps.Any(v => $"Demo1APIv{v.ToString()}" == docName));
                });
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            app.UseSwagger();
            app.UseSwaggerUI(c =>
            {
                c.SwaggerEndpoint($"/swagger/Demo1APIv1/swagger.json", $"测试1APIv1");
                c.SwaggerEndpoint($"/swagger/Demo1APIv2/swagger.json", $"测试1APIv2");
                c.RoutePrefix = string.Empty;//这里主要是不需要再输入swagger这个默认前缀
            });
            app.UseMvc();
        }
    }
}
